/* $Id: fixfold.c$
 *
 * Program to salvage messages from a folder file.
 */
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>

int
main (int argc, char **argv)
{
    struct stat st;
    FILE *inf, *outf;
    long artSize, getOffset, nextOffset;

    if (argc != 3) {
	fputs("Salvage messages from Yarn folder file.\n", stderr);
	fputs("usage: fixfold infile outfile\n", stderr);
	exit(EXIT_FAILURE);
    }

    if (stat(argv[1], &st) != 0) {
	perror(argv[1]);
	exit(EXIT_FAILURE);
    }

    if ((inf = fopen(argv[1], "rb")) == NULL) {
	perror(argv[1]);
	exit(EXIT_FAILURE);
    }
    
    if ((outf = fopen(argv[2], "ab")) == NULL) {
	perror(argv[2]);
	exit(EXIT_FAILURE);
    }

    while (fread(&artSize, sizeof(artSize), 1, inf) == 1) {
	getOffset = ftell(inf);
	nextOffset = getOffset + artSize;

	if (artSize < 0 || nextOffset > st.st_size) {
	    int c;

	    // The article size is bogus.
	    printf("Message at offset %lu has size %lu.  ",
		   getOffset - sizeof(artSize), artSize);
	    fflush(stdout);

	    // Search for next article that has size less than 65536 bytes.
	    do {
		do {
		    c = fgetc(inf);
		} while (c != EOF && c != '\0');
		if (c != EOF)
		    c = fgetc(inf);
	    } while (c != EOF && c != '\0');

	    printf("Skipped %ld bytes.\n", ftell(inf) - getOffset);
	    if (c == EOF)
		break;
	    fseek(inf, -sizeof(artSize), SEEK_CUR);
	    continue;
	}

	// Copy message to other folder.
	fwrite(&artSize, sizeof(artSize), 1, outf);

	while (artSize > 0) {
	    char buf[BUFSIZ];
	    unsigned nread, count;

	    nread = (artSize < BUFSIZ) ? (unsigned)artSize : BUFSIZ;
	    count = fread(buf, sizeof(char), nread, inf);
	    if (count == 0) {
		fprintf(stderr, "errno %d reading message\n", errno);
		return 0;
	    }
	    if (fwrite(buf, sizeof(char), count, outf) != count) {
		fprintf(stderr, "errno %d writing message\n", errno);
		return 0;
	    }
	    artSize -= count;
	}
    }

    fclose(outf);
    fclose(inf);

    return EXIT_SUCCESS;
}
